@@ -53,6 +53,10 @@ class CommandShell
     "shell"
   end
 
+  def self.can_cleanup_files
+    true
+  end
+
   def initialize(conn, opts = {})
     self.platform ||= ""
     self.arch     ||= ""
@@ -107,18 +111,16 @@ class CommandShell
           banner.gsub!(/[^[:print:][:space:]]+/n, "_")
           banner.strip!
 
-          banner = %Q{
+          session_info = @banner = %Q{
 Shell Banner:
 #{banner}
 -----
           }
-
-          session_info = banner
         end
       end
 
       token = Rex::Text.rand_text_alphanumeric(8..24)
-      response = shell_command("echo #{token}", 3)
+      response = shell_command("echo #{token}")
       unless response&.include?(token)
         dlog("Session #{session.sid} failed to respond to an echo command")
         print_error("Command shell session #{session.sid} is not valid and will be closed")
@@ -648,8 +650,13 @@ Shell Banner:
     # Do nil check for cmd (CTRL+D will cause nil error)
     return unless cmd
 
-    arguments = Shellwords.shellwords(cmd)
-    method    = arguments.shift
+    begin
+      arguments = Shellwords.shellwords(cmd)
+      method = arguments.shift
+    rescue ArgumentError => e
+      # Handle invalid shellwords, such as unmatched quotes
+      # See https://github.com/rapid7/metasploit-framework/issues/15912
+    end
 
     # Built-in command
     if commands.key?(method)
@@ -781,6 +788,7 @@ Shell Banner:
   attr_accessor :arch
   attr_accessor :platform
   attr_accessor :max_threads
+  attr_reader :banner
 
 protected
 
@@ -804,7 +812,7 @@ protected
 
     # Displays +info+ on all session startups
     # +info+ is set to the shell banner and initial prompt in the +bootstrap+ method
-    user_output.print("#{self.info}\n") if (self.info && !self.info.empty?) && self.interacting
+    user_output.print("#{@banner}\n") if !@banner.blank? && self.interacting
 
     run_single('')
 
