@@ -704,7 +704,6 @@ class Exploit < Msf::Module
   #
   def is_payload_compatible?(name)
     p = framework.payloads[name]
-
     return false unless p
 
     # Skip over payloads that are too big
@@ -713,7 +712,7 @@ class Exploit < Msf::Module
     begin
       pi = p.new
     rescue ::Exception, ::LoadError => e
-      wlog("Module #{name} failed to initialize: #{e}", 'core', LEV_0)
+      wlog("Module #{name} failed to initialize payload when checking exploit compatibility: #{e}", 'core', LEV_0)
       return false
     end
 
@@ -721,10 +720,6 @@ class Exploit < Msf::Module
     # what not?
     return false if !compatible?(pi)
 
-    if self.needs_cleanup && !pi.can_cleanup
-      return false
-    end
-
     # If the payload is privileged but the exploit does not give
     # privileged access, then fail it.
     return false if !self.privileged && pi.privileged
