@@ -35,7 +35,7 @@ module Exploit::Remote::HttpClient
     register_advanced_options(
       [
         OptString.new('UserAgent', [false, 'The User-Agent header to use for all requests',
-          Rex::Proto::Http::Client::DefaultUserAgent
+          Rex::UserAgent.session_agent
           ]),
         OptString.new('HttpUsername', [false, 'The HTTP username to specify for authentication', '']),
         OptString.new('HttpPassword', [false, 'The HTTP password to specify for authentication', '']),
@@ -47,7 +47,8 @@ module Exploit::Remote::HttpClient
         OptFloat.new('HttpClientTimeout', [false, 'HTTP connection and receive timeout']),
         OptBool.new('HttpTrace', [false, 'Show the raw HTTP requests and responses', false]),
         OptBool.new('HttpTraceHeadersOnly', [false, 'Show HTTP headers only in HttpTrace', false]),
-        OptString.new('HttpTraceColors', [false, 'HTTP request and response colors for HttpTrace (unset to disable)', 'red/blu'])
+        OptString.new('HttpTraceColors', [false, 'HTTP request and response colors for HttpTrace (unset to disable)', 'red/blu']),
+        OptString.new('SSLServerNameIndication', [ false, 'SSL/TLS Server Name Indication (SNI)', nil]),
       ], self.class
     )
 
@@ -73,6 +74,8 @@ module Exploit::Remote::HttpClient
         OptInt.new('HTTP::pad_get_params_count', [false, 'How many fake query string variables to insert into the request', 16]),
         OptBool.new('HTTP::pad_post_params', [false, 'Insert random, fake post variables into the request', false]),
         OptInt.new('HTTP::pad_post_params_count', [false, 'How many fake post variables to insert into the request', 16]),
+        OptBool.new('HTTP::shuffle_get_params', [false, 'Randomize order of GET parameters', false]),
+        OptBool.new('HTTP::shuffle_post_params', [false, 'Randomize order of POST parameters', false]),
         OptBool.new('HTTP::uri_fake_end', [false, 'Add a fake end of URI (eg: /%20HTTP/1.0/../../)', false]),
         OptBool.new('HTTP::uri_fake_params_start', [false, 'Add a fake start of params to the URI (eg: /%3fa=b/../)', false]),
         OptBool.new('HTTP::header_folding', [false, 'Enable folding of HTTP headers', false])
@@ -91,7 +94,7 @@ module Exploit::Remote::HttpClient
   end
 
   def deregister_http_client_options
-    deregister_options('RHOST', 'RPORT', 'VHOST', 'SSL', 'Proxies')
+    deregister_options('RHOST', 'RPORT', 'VHOST', 'SSL', 'SSLServerNameIndication', 'Proxies')
   end
 
   #
@@ -134,7 +137,6 @@ module Exploit::Remote::HttpClient
     end
   end
 
-
   #
   # Connects to an HTTP server.
   #
@@ -160,12 +162,16 @@ module Exploit::Remote::HttpClient
       ssl_version,
       proxies,
       client_username,
-      client_password
+      client_password,
+      comm: opts['comm']
     )
 
+
     # Configure the HTTP client with the supplied parameter
+    vhost = opts['vhost'] || opts['rhost'] || self.vhost
     nclient.set_config(
-      'vhost' => opts['vhost'] || opts['rhost'] || self.vhost(),
+      'vhost' => vhost,
+      'ssl_server_name_indication' => datastore['SSLServerNameIndication'] || vhost,
       'agent' => datastore['UserAgent'],
       'partial' => opts['partial'],
       'uri_encode_mode'        => datastore['HTTP::uri_encode_mode'],
@@ -188,6 +194,8 @@ module Exploit::Remote::HttpClient
       'pad_get_params_count'   => datastore['HTTP::pad_get_params_count'],
       'pad_post_params'        => datastore['HTTP::pad_post_params'],
       'pad_post_params_count'  => datastore['HTTP::pad_post_params_count'],
+      'shuffle_get_params'     => datastore['HTTP::shuffle_get_params'],
+      'shuffle_post_params'    => datastore['HTTP::shuffle_post_params'],
       'uri_fake_end'           => datastore['HTTP::uri_fake_end'],
       'uri_fake_params_start'  => datastore['HTTP::uri_fake_params_start'],
       'header_folding'         => datastore['HTTP::header_folding'],
@@ -217,6 +225,42 @@ module Exploit::Remote::HttpClient
     return nclient
   end
 
+  #
+  # Establish a WebSocket connection to the remote server.
+  #
+  # @raise [Rex::Proto::Http::WebSocket::WebSocketError] raises an exception if the connection fails
+  # @return [Rex::Proto::Http::WebSocket::Interface]
+  def connect_ws(opts={}, timeout = 20)
+    ws_key = Rex::Text.rand_text_alphanumeric(20)
+    opts['headers'] = opts.fetch('headers', {}).merge({
+      'Connection' => 'Upgrade',
+      'Upgrade' => 'WebSocket',
+      'Sec-WebSocket-Version' => 13,
+      'Sec-WebSocket-Key' => ws_key
+    })
+
+    if (http_client = opts['client']).nil?
+      opts['client'] = http_client = connect(opts)
+      raise Rex::Proto::Http::WebSocket::ConnectionError.new if http_client.nil?
+    end
+
+    res = send_request_raw(opts, timeout, false)
+    unless res&.code == 101
+      disconnect
+      raise Rex::Proto::Http::WebSocket::ConnectionError.new(http_response: res)
+    end
+
+    # see: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Sec-WebSocket-Accept
+    accept_ws_key = Rex::Text.encode_base64(OpenSSL::Digest::SHA1.digest(ws_key + '258EAFA5-E914-47DA-95CA-C5AB0DC85B11'))
+    unless res.headers['Sec-WebSocket-Accept'] == accept_ws_key
+      disconnect
+      raise Rex::Proto::Http::WebSocket::ConnectionError.new(msg: 'Invalid Sec-WebSocket-Accept header', http_response: res)
+    end
+
+    socket = http_client.conn
+    socket.extend(Rex::Proto::Http::WebSocket::Interface)
+  end
+
   #
   # Converts datastore options into configuration parameters for the
   # Metasploit::LoginScanner::Http class. Any parameters passed into
@@ -253,6 +297,8 @@ module Exploit::Remote::HttpClient
       evade_pad_get_params_count:    datastore['HTTP::pad_get_params_count'],
       evade_pad_post_params:         datastore['HTTP::pad_post_params'],
       evade_pad_post_params_count:   datastore['HTTP::pad_post_params_count'],
+      evade_shuffle_get_params:      datastore['HTTP::shuffle_get_params'],
+      evade_shuffle_post_params:     datastore['HTTP::shuffle_post_params'],
       evade_uri_fake_end:            datastore['HTTP::uri_fake_end'],
       evade_uri_fake_params_start:   datastore['HTTP::uri_fake_params_start'],
       evade_header_folding:          datastore['HTTP::header_folding'],
@@ -320,7 +366,7 @@ module Exploit::Remote::HttpClient
       actual_timeout = opts[:timeout] || timeout
     end
 
-    c = connect(opts)
+    c = opts['client'] || connect(opts)
     r = opts[:cgi] ? c.request_cgi(opts) : c.request_raw(opts)
 
     if datastore['HttpTrace']
@@ -450,7 +496,8 @@ module Exploit::Remote::HttpClient
       end
 
       opts['rhost'] = datastore['RHOST']
-      opts['vhost'] = opts['vhost'] || opts['rhost'] || self.vhost()
+      opts['vhost'] = opts['vhost'] || self.vhost() || opts['rhost']
+      opts['ssl_server_name_indication'] = datastore['SSLServerNameIndication'] || opts['vhost']
       opts['rport'] = datastore['RPORT']
 
       opts['SSL'] = ssl
@@ -461,6 +508,7 @@ module Exploit::Remote::HttpClient
       opts['uri'] = location.path
       opts['rhost'] = location.host
       opts['vhost'] = location.host
+      opts['ssl_server_name_indication'] = opts['vhost']
       opts['rport'] = location.port
 
       if location.scheme == 'https'
