@@ -45,9 +45,9 @@ module Msf
         register_advanced_options(
         [
           OptBool.new('SMBDirect', [ false, 'The target port is a raw SMB service (not NetBIOS)', true ]),
-          OptString.new('SMBUser', [ false, 'The username to authenticate as', '']),
-          OptString.new('SMBPass', [ false, 'The password for the specified username', '']),
-          OptString.new('SMBDomain',  [ false, 'The Windows domain to use for authentication', '.']),
+          OptString.new('SMBUser', [ false, 'The username to authenticate as', ''], fallbacks: ['USERNAME']),
+          OptString.new('SMBPass', [ false, 'The password for the specified username', ''], fallbacks: ['PASSWORD']),
+          OptString.new('SMBDomain',  [ false, 'The Windows domain to use for authentication', '.'], fallbacks: ['DOMAIN']),
           OptString.new('SMBName', [ true, 'The NetBIOS hostname (required for port 139 connections)', '*SMBSERVER']),
           OptBool.new('SMB::VerifySignature', [ true, "Enforces client-side verification of server response signatures", false]),
           OptInt.new('SMB::ChunkSize', [ true, 'The chunk size for SMB segments, bigger values will increase speed but break NT 4.0 and SMB signing', 500]),
