@@ -1,6 +1,7 @@
 # -*- coding: binary -*-
 # frozen_string_literal: true
 
+require 'rex/text'
 
 module Msf
   ###
@@ -14,7 +15,9 @@ module Msf
 
     CONFIG_KEY = 'framework/features'
     WRAPPED_TABLES = 'wrapped_tables'
+    DATASTORE_FALLBACKS = 'datastore_fallbacks'
     FULLY_INTERACTIVE_SHELLS = 'fully_interactive_shells'
+    SERVICEMANAGER_COMMAND = 'servicemanager_command'
     DEFAULTS = [
       {
         name: WRAPPED_TABLES,
@@ -25,6 +28,17 @@ module Msf
         name: FULLY_INTERACTIVE_SHELLS,
         description: 'When enabled you will have the option to drop into a fully interactive shell from within meterpreter',
         default_value: false
+      }.freeze,
+      {
+        name: SERVICEMANAGER_COMMAND,
+        description: 'When enabled you will have access to the _servicemanager command',
+        default_value: false
+      }.freeze,
+      {
+        name: DATASTORE_FALLBACKS,
+        description: 'When enabled you can consistently set username across modules, instead of setting SMBUser/FTPUser/BIND_DN/etc',
+        requires_restart: true,
+        default_value: false
       }.freeze
     ].freeze
 
@@ -52,6 +66,8 @@ module Msf
       end
     end
 
+    # @param [String] name The feature name
+    # @return [TrueClass,FalseClass] True if the flag is be enabled, false otherwise
     def enabled?(name)
       return false unless @flag_lookup[name]
 
@@ -59,6 +75,14 @@ module Msf
       feature.key?(:user_preference) ? feature[:user_preference] : feature[:default_value]
     end
 
+    # @param [String] name The feature name
+    # @return [TrueClass,FalseClass] True if the flag requires a console restart to work effectively
+    def requires_restart?(name)
+      return false unless @flag_lookup[name]
+
+      @flag_lookup[name][:requires_restart] == true
+    end
+
     def exists?(name)
       @flag_lookup.key?(name)
     end
