@@ -101,46 +101,70 @@ class Db
     true
   end
 
+  @@workspace_opts = Rex::Parser::Arguments.new(
+    [ '-h', '--help' ] => [ false, 'Help banner.'],
+    [ '-a', '--add' ] => [ true, 'Add a workspace.', '<name>'],
+    [ '-d', '--delete' ] => [ true, 'Delete a workspace.', '<name>'],
+    [ '-D', '--delete-all' ] => [ false, 'Delete all workspaces.'],
+    [ '-r', '--rename' ] => [ true, 'Rename a workspace.', '<old> <new>'],
+    [ '-l', '--list' ] => [ false, 'List workspaces.'],
+    [ '-v', '--list-verbose' ] => [ false, 'List workspaces verbosely.'],
+    [ '-S', '--search' ] => [ true, 'Search for a workspace.', '<name>']
+  )
+
   def cmd_workspace_help
     print_line "Usage:"
-    print_line "    workspace                  List workspaces"
-    print_line "    workspace -v               List workspaces verbosely"
-    print_line "    workspace [name]           Switch workspace"
-    print_line "    workspace -a [name] ...    Add workspace(s)"
-    print_line "    workspace -d [name] ...    Delete workspace(s)"
-    print_line "    workspace -D               Delete all workspaces"
-    print_line "    workspace -r <old> <new>   Rename workspace"
-    print_line "    workspace -h               Show this help information"
-    print_line
+    print_line "    workspace          List workspaces"
+    print_line "    workspace [name]   Switch workspace"
+    print_line @@workspace_opts.usage
   end
 
   def cmd_workspace(*args)
     return unless active?
+
+    state = :nil
+
+    list = false
+    verbose = false
+    names = []
     search_term = nil
-    while (arg = args.shift)
-      case arg
-      when '-h','--help'
+
+    @@workspace_opts.parse(args) do |opt, idx, val|
+      case opt
+      when '-h', '--help'
         cmd_workspace_help
         return
-      when '-a','--add'
-        adding = true
-      when '-d','--del'
-        deleting = true
-      when '-D','--delete-all'
-        delete_all = true
-      when '-r','--rename'
-        renaming = true
-      when '-v','--verbose'
+      when '-a', '--add'
+        return cmd_workspace_help unless state == :nil
+
+        state = :adding
+        names << val if !val.nil?
+      when '-d', '--del'
+        return cmd_workspace_help unless state == :nil
+
+        state = :deleting
+        names << val if !val.nil?
+      when '-D', '--delete-all'
+        return cmd_workspace_help unless state == :nil
+
+        state = :delete_all
+      when '-r', '--rename'
+        return cmd_workspace_help unless state == :nil
+
+        state = :renaming
+        names << val if !val.nil?
+      when '-v', '--verbose'
         verbose = true
+      when '-l', '--list'
+        list = true
       when '-S', '--search'
-        search_term = args.shift
+        search_term = val
       else
-        names ||= []
-        names << arg
+        names << val if !val.nil?
       end
     end
 
-    if adding and names
+    if state == :adding and names
       # Add workspaces
       wspace = nil
       names.each do |name|
@@ -154,7 +178,7 @@ class Db
       end
       framework.db.workspace = wspace
       print_status("Workspace: #{framework.db.workspace.name}")
-    elsif deleting and names
+    elsif state == :deleting and names
       ws_ids_to_delete = []
       starting_ws = framework.db.workspace
       names.uniq.each do |n|
@@ -167,7 +191,7 @@ class Db
       else
         print_status("No workspaces matching the given name(s) were found.")
       end
-    elsif delete_all
+    elsif state == :delete_all
       ws_ids_to_delete = []
       starting_ws = framework.db.workspace
       framework.db.workspaces.each do |ws|
@@ -175,7 +199,7 @@ class Db
       end
       deleted = framework.db.delete_workspaces(ids: ws_ids_to_delete)
       process_deleted_workspaces(deleted, starting_ws)
-    elsif renaming
+    elsif state == :renaming
       if names.length != 2
         print_error("Wrong number of arguments to rename")
         return
@@ -207,7 +231,7 @@ class Db
         print_error "Failed to rename workspace: #{e.message}"
       end
 
-    elsif names
+    elsif !names.empty?
       name = names.last
       # Switch workspace
       workspace = framework.db.find_workspace(name)
@@ -275,7 +299,32 @@ class Db
 
   def cmd_workspace_tabs(str, words)
     return [] unless active?
-    framework.db.workspaces.map { |s| s.name } if (words & ['-a','--add']).empty?
+    framework.db.workspaces.map(&:name) if (words & ['-a','--add']).empty?
+  end
+
+  #
+  # Tab completion for the hosts command
+  #
+  # @param str [String] the string currently being typed before tab was hit
+  # @param words [Array<String>] the previously completed words on the command line.  words is always
+  # at least 1 when tab completion has reached this stage since the command itself has been completed
+  def cmd_hosts_tabs(str, words)
+    if words.length == 1
+      return @@hosts_opts.option_keys.select { |opt| opt.start_with?(str) }
+    end
+
+    case words[-1]
+    when '-c', '--columns', '-C', '--columns-until-restart'
+      return @@hosts_columns
+    when '-o', '--output'
+      return tab_complete_filenames(str, words)
+    end
+
+    if @@hosts_opts.arg_required?(words[-1])
+      return []
+    end
+
+    return @@hosts_opts.option_keys.select { |opt| opt.start_with?(str) }
   end
 
   def cmd_hosts_help
@@ -364,6 +413,33 @@ class Db
 
   @@hosts_columns = [ 'address', 'mac', 'name', 'os_name', 'os_flavor', 'os_sp', 'purpose', 'info', 'comments']
 
+  @@hosts_opts = Rex::Parser::Arguments.new(
+    [ '-h', '--help' ] => [ false, 'Show this help information' ],
+    [ '-a', '--add' ] => [ true, 'Add the hosts instead of searching', '<host>' ],
+    [ '-u', '--up' ] => [ false, 'Only show hosts which are up' ],
+    [ '-R', '--rhosts' ] => [ false, 'Set RHOSTS from the results of the search' ],
+    [ '-S', '--search' ] => [ true, 'Search string to filter by', '<filter>' ],
+    [ '-i', '--info' ] => [ true, 'Change the info of a host', '<info>' ],
+    [ '-n', '--name' ] => [ true, 'Change the name of a host', '<name>' ],
+    [ '-m', '--comment' ] => [ true, 'Change the comment of a host', '<comment>' ],
+    [ '-t', '--tag' ] => [ true, 'Add or specify a tag to a range of hosts', '<tag>' ],
+    [ '-T', '--delete-tag' ] => [ true, 'Remove a tag from a range of hosts', '<tag>' ],
+    [ '-d', '--delete' ] => [ true, 'Delete the hosts instead of searching', '<hosts>' ],
+    [ '-o', '--output' ] => [ true, 'Send output to a file in csv format', '<filename>' ],
+    [ '-O', '--order' ] => [ true, 'Order rows by specified column number', '<column id>' ],
+    [ '-c', '--columns' ] => [ true, 'Only show the given columns (see list below)', '<columns>' ],
+    [ '-C', '--columns-until-restart' ] => [ true, 'Only show the given columns until the next restart (see list below)', '<columns>' ],
+  )
+
+  def cmd_hosts_help(default_columns)
+    print_line "Usage: hosts [ options ] [addr1 addr2 ...]"
+    print_line
+    print @@hosts_opts.usage
+    print_line
+    print_line "Available columns: #{default_columns.join(", ")}"
+    print_line
+  end
+
   def cmd_hosts(*args)
     return unless active?
     onlyup = false
@@ -375,6 +451,12 @@ class Db
     host_ranges = []
     search_term = nil
 
+    order_by = nil
+    info_data = nil
+    name_data = nil
+    comment_data = nil
+    tag_name = nil
+
     output = nil
     default_columns = [
         'address',
@@ -410,14 +492,42 @@ class Db
     col_search = @@hosts_columns
 
     default_columns.delete_if {|v| (v[-2,2] == "id")}
-    while (arg = args.shift)
-      case arg
-      when '-a','--add'
+    @@hosts_opts.parse(args) do |opt, idx, val|
+      case opt
+      when '-h', '--help'
+        cmd_hosts_help(default_columns)
+        return
+      when '-a', '--add'
         mode << :add
-      when '-d','--delete'
+        arg_host_range(val, host_ranges)
+      when '-d', '--delete'
         mode << :delete
-      when '-c','-C'
-        list = args.shift
+        arg_host_range(val, host_ranges)
+      when '-u', '--up'
+        onlyup = true
+      when '-o'
+        output = val
+      when '-R', '--rhosts'
+        set_rhosts = true
+      when '-S', '--search'
+        search_term = val
+      when '-i', '--info'
+        mode << :new_info
+        info_data = val
+      when '-n', '--name'
+        mode << :new_name
+        name_data = val
+      when '-m', '--comment'
+        mode << :new_comment
+        comment_data = val
+      when '-t', '--tag'
+        mode << :tag
+        tag_name = val
+      when '-T', '--delete-tag'
+        mode << :delete_tag
+        tag_name = val
+      when '-c', '-C'
+        list = val
         if(!list)
           print_error("Invalid column list")
           return
@@ -430,60 +540,17 @@ class Db
             return
           end
         }
-        if (arg == '-C')
+        if opt == '-C'
           @@hosts_columns = col_search
         end
-
-      when '-u','--up'
-        onlyup = true
-      when '-o'
-        output = args.shift
       when '-O'
-        if (order_by = args.shift.to_i - 1) < 0
+        if (order_by = val.to_i - 1) < 0
           print_error('Please specify a column number starting from 1')
           return
         end
-      when '-R', '--rhosts'
-        set_rhosts = true
-      when '-S', '--search'
-        search_term = args.shift
-      when '-i', '--info'
-        mode << :new_info
-        info_data = args.shift
-      when '-n', '--name'
-        mode << :new_name
-        name_data = args.shift
-      when '-m', '--comment'
-        mode << :new_comment
-        comment_data = args.shift
-      when '-t', '--tag'
-        mode << :tag
-        tag_name = args.shift
-      when '-h','--help'
-        print_line "Usage: hosts [ options ] [addr1 addr2 ...]"
-        print_line
-        print_line "OPTIONS:"
-        print_line "  -a,--add          Add the hosts instead of searching"
-        print_line "  -d,--delete       Delete the hosts instead of searching"
-        print_line "  -c <col1,col2>    Only show the given columns (see list below)"
-        print_line "  -C <col1,col2>    Only show the given columns until the next restart (see list below)"
-        print_line "  -h,--help         Show this help information"
-        print_line "  -u,--up           Only show hosts which are up"
-        print_line "  -o <file>         Send output to a file in csv format"
-        print_line "  -O <column>       Order rows by specified column number"
-        print_line "  -R,--rhosts       Set RHOSTS from the results of the search"
-        print_line "  -S,--search       Search string to filter by"
-        print_line "  -i,--info         Change the info of a host"
-        print_line "  -n,--name         Change the name of a host"
-        print_line "  -m,--comment      Change the comment of a host"
-        print_line "  -t,--tag          Add or specify a tag to a range of hosts"
-        print_line
-        print_line "Available columns: #{default_columns.join(", ")}"
-        print_line
-        return
       else
         # Anything that wasn't an option is a host to search for
-        unless (arg_host_range(arg, host_ranges))
+        unless (arg_host_range(val, host_ranges))
           return
         end
       end
@@ -544,8 +611,16 @@ class Db
         end
       end
       return
-    when mode.include?(:tag) && mode.include?(:delete)
-      delete_host_tag(host_ranges, tag_name)
+    when mode == [:delete_tag]
+      begin
+        delete_host_tag(host_ranges, tag_name)
+      rescue => e
+        if e.message.include?('Validation failed')
+          print_error(e.message)
+        else
+          raise e
+        end
+      end
       return
     end
 
@@ -564,9 +639,8 @@ class Db
             when "workspace"; host.workspace.name
             when "tags"
               found_tags = find_host_tags(framework.db.workspace, host.id)
-              tag_names = []
-              found_tags.each { |t| tag_names << t.name }
-              found_tags * ", "
+              tag_names = found_tags.map(&:name).join(', ')
+              tag_names
             end
           # Otherwise, it's just an attribute
           else
@@ -604,12 +678,58 @@ class Db
     print_status("Deleted #{delete_count} hosts") if delete_count > 0
   end
 
-  def cmd_services_help
-    # Like cmd_hosts, use "-h" instead of recreating the column list
-    # here
-    cmd_services("-h")
+  #
+  # Tab completion for the services command
+  #
+  # @param str [String] the string currently being typed before tab was hit
+  # @param words [Array<String>] the previously completed words on the command line.  words is always
+  # at least 1 when tab completion has reached this stage since the command itself has been completed
+  def cmd_services_tabs(str, words)
+    if words.length == 1
+      return @@services_opts.option_keys.select { |opt| opt.start_with?(str) }
+    end
+
+    case words[-1]
+    when '-c', '--column'
+      return @@services_columns
+    when '-O', '--order'
+      return []
+    when '-p', '--port'
+      return []
+    when '-r', '--protocol'
+      return []
+    end
+
+    []
   end
 
+  def cmd_services_help(default_columns)
+    print_line "Usage: services [-h] [-u] [-a] [-r <proto>] [-p <port1,port2>] [-s <name1,name2>] [-o <filename>] [addr1 addr2 ...]"
+    print_line
+    print @@services_opts.usage
+    print_line
+    print_line "Available columns: #{default_columns.join(", ")}"
+    print_line
+  end
+
+  @@services_columns = [ 'created_at', 'info', 'name', 'port', 'proto', 'state', 'updated_at' ]
+
+  @@services_opts = Rex::Parser::Arguments.new(
+    [ '-a', '--add' ] => [ false, 'Add the services instead of searching.' ],
+    [ '-d', '--delete' ] => [ false, 'Delete the services instead of searching.' ],
+    [ '-U', '--update' ] => [ false, 'Update data for existing service.' ],
+    [ '-u', '--up' ] => [ false, 'Only show services which are up.' ],
+    [ '-c', '--column' ] => [ true, 'Only show the given columns.', '<col1,col2>' ],
+    [ '-p', '--port' ] => [ true, 'Search for a list of ports.', '<ports>' ],
+    [ '-r', '--protocol' ] => [ true, 'Protocol type of the service being added [tcp|udp].', '<protocol>' ],
+    [ '-s', '--name' ] => [ true, 'Name of the service to add.', '<name>' ],
+    [ '-o', '--output' ] => [ true, 'Send output to a file in csv format.', '<filename>' ],
+    [ '-O', '--order' ] => [ true, 'Order rows by specified column number.', '<column id>' ],
+    [ '-R', '--rhosts' ] => [ false, 'Set RHOSTS from the results of the search.' ],
+    [ '-S', '--search' ] => [ true, 'Search string to filter by.', '<filter>' ],
+    [ '-h', '--help' ] => [ false, 'Show this help information.' ]
+  )
+
   def cmd_services(*args)
     return unless active?
     mode = :search
@@ -617,15 +737,10 @@ class Db
     output_file = nil
     set_rhosts = false
     col_search = ['port', 'proto', 'name', 'state', 'info']
-    default_columns = [
-        'created_at',
-        'info',
-        'name',
-        'port',
-        'proto',
-        'state',
-        'updated_at']
 
+    names = nil
+    order_by = nil
+    proto = nil
     host_ranges  = []
     port_ranges  = []
     rhosts       = []
@@ -633,91 +748,72 @@ class Db
     search_term  = nil
     opts         = {}
 
-    # option parsing
-    while (arg = args.shift)
-      case arg
-        when '-a','--add'
-          mode = :add
-        when '-d','--delete'
-          mode = :delete
-        when '-U', '--update'
-          mode = :update
-        when '-u','--up'
-          onlyup = true
-        when '-c'
-          list = args.shift
-          if(!list)
-            print_error("Invalid column list")
-            return
-          end
-          col_search = list.strip().split(",")
-          col_search.each { |c|
-            if not default_columns.include? c
-              print_error("Invalid column list. Possible values are (#{default_columns.join("|")})")
-              return
-            end
-          }
-        when '-p'
-          unless (arg_port_range(args.shift, port_ranges, true))
-            return
-          end
-        when '-r'
-          proto = args.shift
-          if (!proto)
-            print_status("Invalid protocol")
-            return
-          end
-          proto = proto.strip
-        when '-s'
-          namelist = args.shift
-          if (!namelist)
-            print_error("Invalid name list")
-            return
-          end
-          names = namelist.strip().split(",")
-        when '-o'
-          output_file = args.shift
-          if (!output_file)
-            print_error("Invalid output filename")
-            return
-          end
-          output_file = ::File.expand_path(output_file)
-        when '-O'
-          if (order_by = args.shift.to_i - 1) < 0
-            print_error('Please specify a column number starting from 1')
-            return
-          end
-        when '-R', '--rhosts'
-          set_rhosts = true
-        when '-S', '--search'
-          search_term = args.shift
-          opts[:search_term] = search_term
-        when '-h','--help'
-          print_line
-          print_line "Usage: services [-h] [-u] [-a] [-r <proto>] [-p <port1,port2>] [-s <name1,name2>] [-o <filename>] [addr1 addr2 ...]"
-          print_line
-          print_line "  -a,--add          Add the services instead of searching"
-          print_line "  -d,--delete       Delete the services instead of searching"
-          print_line "  -c <col1,col2>    Only show the given columns"
-          print_line "  -h,--help         Show this help information"
-          print_line "  -s <name>         Name of the service to add"
-          print_line "  -p <port>         Search for a list of ports"
-          print_line "  -r <protocol>     Protocol type of the service being added [tcp|udp]"
-          print_line "  -u,--up           Only show services which are up"
-          print_line "  -o <file>         Send output to a file in csv format"
-          print_line "  -O <column>       Order rows by specified column number"
-          print_line "  -R,--rhosts       Set RHOSTS from the results of the search"
-          print_line "  -S,--search       Search string to filter by"
-          print_line "  -U,--update       Update data for existing service"
-          print_line
-          print_line "Available columns: #{default_columns.join(", ")}"
-          print_line
+    @@services_opts.parse(args) do |opt, idx, val|
+      case opt
+      when '-a', '--add'
+        mode = :add
+      when '-d', '--delete'
+        mode = :delete
+      when '-U', '--update'
+        mode = :update
+      when '-u', '--up'
+        onlyup = true
+      when '-c'
+        list = val
+        if(!list)
+          print_error("Invalid column list")
           return
-        else
-          # Anything that wasn't an option is a host to search for
-          unless (arg_host_range(arg, host_ranges))
+        end
+        col_search = list.strip().split(",")
+        col_search.each { |c|
+          if not @@services_columns.include? c
+            print_error("Invalid column list. Possible values are (#{@@services_columns.join("|")})")
             return
           end
+        }
+      when '-p'
+        unless (arg_port_range(val, port_ranges, true))
+          return
+        end
+      when '-r'
+        proto = val
+        if (!proto)
+          print_status("Invalid protocol")
+          return
+        end
+        proto = proto.strip
+      when '-s'
+        namelist = val
+        if (!namelist)
+          print_error("Invalid name list")
+          return
+        end
+        names = namelist.strip().split(",")
+      when '-o'
+        output_file = val
+        if (!output_file)
+          print_error("Invalid output filename")
+          return
+        end
+        output_file = ::File.expand_path(output_file)
+      when '-O'
+        if (order_by = val.to_i - 1) < 0
+          print_error('Please specify a column number starting from 1')
+          return
+        end
+      when '-R', '--rhosts'
+        set_rhosts = true
+      when '-S', '--search'
+        search_term = val
+        opts[:search_term] = search_term
+      when '-h', '--help'
+        cmd_services_help(@@services_columns)
+        return
+      else
+        # Anything that wasn't an option is a host to search for
+        unless (arg_host_range(val, host_ranges))
+          return
+        end
       end
     end
 
@@ -751,7 +847,7 @@ class Db
     end
 
     # If we got here, we're searching.  Delete implies search
-    col_names = default_columns
+    col_names = @@services_columns
     if col_search
       col_names = col_search
     end
@@ -817,18 +913,24 @@ class Db
 
   end
 
+  #
+  # Tab completion for the vulns command
+  #
+  # @param str [String] the string currently being typed before tab was hit
+  # @param words [Array<String>] the previously completed words on the command line.  words is always
+  # at least 1 when tab completion has reached this stage since the command itself has been completed
+  def cmd_vulns_tabs(str, words)
+    if words.length == 1
+      return @@vulns_opts.option_keys.select { |opt| opt.start_with?(str) }
+    end
+  end
+
   def cmd_vulns_help
     print_line "Print all vulnerabilities in the database"
     print_line
     print_line "Usage: vulns [addr range]"
     print_line
-    print_line "  -h,--help             Show this help information"
-    print_line "  -o <file>             Send output to a file in csv format"
-    print_line "  -p,--port <portspec>  List vulns matching this port spec"
-    print_line "  -s <svc names>        List vulns matching these service names"
-    print_line "  -R,--rhosts           Set RHOSTS from the results of the search"
-    print_line "  -S,--search           Search string to filter by"
-    print_line "  -i,--info             Display vuln information"
+    print @@vulns_opts.usage
     print_line
     print_line "Examples:"
     print_line "  vulns -p 1-65536          # only vulns with associated services"
@@ -836,6 +938,17 @@ class Db
     print_line
   end
 
+  @@vulns_opts = Rex::Parser::Arguments.new(
+    [ '-h', '--help' ] => [ false, 'Show this help information.' ],
+    [ '-o', '--output' ] => [ true, 'Send output to a file in csv format.', '<filename>' ],
+    [ '-p', '--port' ] => [ true, 'List vulns matching this port spec.', '<port>' ],
+    [ '-s', '--service' ] => [ true, 'List vulns matching these service names.', '<name>' ],
+    [ '-R', '--rhosts' ] => [ false, 'Set RHOSTS from the results of the search.' ],
+    [ '-S', '--search' ] => [ true, 'Search string to filter by.', '<filter>' ],
+    [ '-i', '--info' ] => [ false, 'Display vuln information.' ],
+    [ '-d', '--delete' ] => [ false, 'Delete vulnerabilities. Not officially supported.' ]
+  )
+
   def cmd_vulns(*args)
     return unless active?
 
@@ -851,17 +964,17 @@ class Db
     output_file = nil
     delete_count = 0
 
-    while (arg = args.shift)
-      case arg
-      # when '-a', '--add'
-      #   mode = :add
-      when '-d', '--delete'  # TODO: This is currently undocumented because it's not officially supported.
+    mode = nil
+
+    @@vulns_opts.parse(args) do |opt, idx, val|
+      case opt
+      when '-d', '--delete' # TODO: This is currently undocumented because it's not officially supported.
         mode = :delete
       when '-h', '--help'
         cmd_vulns_help
         return
       when '-o', '--output'
-        output_file = args.shift
+        output_file = val
         if output_file
           output_file = File.expand_path(output_file)
         else
@@ -869,11 +982,11 @@ class Db
           return
         end
       when '-p', '--port'
-        unless (arg_port_range(args.shift, port_ranges, true))
+        unless (arg_port_range(val, port_ranges, true))
           return
         end
       when '-s', '--service'
-        service = args.shift
+        service = val
         if (!service)
           print_error("Argument required for -s")
           return
@@ -882,12 +995,12 @@ class Db
       when '-R', '--rhosts'
         set_rhosts = true
       when '-S', '--search'
-        search_term = args.shift
+        search_term = val
       when '-i', '--info'
         show_info = true
       else
         # Anything that wasn't an option is a host to search for
-        unless (arg_host_range(arg, host_ranges))
+        unless (arg_host_range(val, host_ranges))
           return
         end
       end
@@ -970,18 +1083,29 @@ class Db
     print_status("Deleted #{delete_count} vulnerabilities") if delete_count > 0
   end
 
+  #
+  # Tab completion for the notes command
+  #
+  # @param str [String] the string currently being typed before tab was hit
+  # @param words [Array<String>] the previously completed words on the command line.  words is always
+  # at least 1 when tab completion has reached this stage since the command itself has been completed
+  def cmd_notes_tabs(str, words)
+    if words.length == 1
+      return @@notes_opts.option_keys.select { |opt| opt.start_with?(str) }
+    end
+
+    case words[-1]
+    when '-O', '--order'
+      return []
+    end
+
+    []
+  end
+
   def cmd_notes_help
     print_line "Usage: notes [-h] [-t <type1,type2>] [-n <data string>] [-a] [addr range]"
     print_line
-    print_line "  -a,--add                  Add a note to the list of addresses, instead of listing"
-    print_line "  -d,--delete               Delete the hosts instead of searching"
-    print_line "  -n,--note <data>          Set the data for a new note (only with -a)"
-    print_line "  -t,--type <type1,type2>   Search for a list of types, or set single type for add"
-    print_line "  -h,--help                 Show this help information"
-    print_line "  -R,--rhosts               Set RHOSTS from the results of the search"
-    print_line "  -S,--search               Search string to filter by"
-    print_line "  -o,--output               Save the notes to a csv file"
-    print_line "  -O <column>               Order rows by specified column number"
+    print @@notes_opts.usage
     print_line
     print_line "Examples:"
     print_line "  notes --add -t apps -n 'winzip' 10.1.1.34 10.1.20.41"
@@ -990,6 +1114,18 @@ class Db
     print_line
   end
 
+  @@notes_opts = Rex::Parser::Arguments.new(
+    [ '-a', '--add' ] => [ false, 'Add a note to the list of addresses, instead of listing.' ],
+    [ '-d', '--delete' ] => [ false, 'Delete the notes instead of searching.' ],
+    [ '-n', '--note' ] => [ true, 'Set the data for a new note (only with -a).', '<note>' ],
+    [ '-t', '--type' ] => [ true, 'Search for a list of types, or set single type for add.', '<type1,type2>' ],
+    [ '-h', '--help' ] => [ false, 'Show this help information.' ],
+    [ '-R', '--rhosts' ] => [ false, 'Set RHOSTS from the results of the search.' ],
+    [ '-o', '--output' ] => [ true, 'Save the notes to a csv file.', '<filename>' ],
+    [ '-O', '--order' ] => [ true, 'Order rows by specified column number.', '<column id>' ],
+    [ '-u', '--update' ] => [ false, 'Update a note. Not officially supported.' ]
+  )
+
   def cmd_notes(*args)
     return unless active?
   ::ApplicationRecord.connection_pool.with_connection {
@@ -1003,21 +1139,22 @@ class Db
     search_term = nil
     output_file = nil
     delete_count = 0
+    order_by = nil
 
-    while (arg = args.shift)
-      case arg
+    @@notes_opts.parse(args) do |opt, idx, val|
+      case opt
       when '-a', '--add'
         mode = :add
       when '-d', '--delete'
         mode = :delete
       when '-n', '--note'
-        data = args.shift
+        data = val
         if(!data)
           print_error("Can't make a note with no data")
           return
         end
       when '-t', '--type'
-        typelist = args.shift
+        typelist = val
         if(!typelist)
           print_error("Invalid type list")
           return
@@ -1026,11 +1163,11 @@ class Db
       when '-R', '--rhosts'
         set_rhosts = true
       when '-S', '--search'
-        search_term = args.shift
+        search_term = val
       when '-o', '--output'
-        output_file = args.shift
+        output_file = val
       when '-O'
-        if (order_by = args.shift.to_i - 1) < 0
+        if (order_by = val.to_i - 1) < 0
           print_error('Please specify a column number starting from 1')
           return
         end
@@ -1041,7 +1178,7 @@ class Db
         return
       else
         # Anything that wasn't an option is a host to search for
-        unless (arg_host_range(arg, host_ranges))
+        unless (arg_host_range(val, host_ranges))
           return
         end
       end
@@ -1195,22 +1332,39 @@ class Db
     end
   end
 
+  #
+  # Tab completion for the loot command
+  #
+  # @param str [String] the string currently being typed before tab was hit
+  # @param words [Array<String>] the previously completed words on the command line.  words is always
+  # at least 1 when tab completion has reached this stage since the command itself has been completed
+  def cmd_loot_tabs(str, words)
+    if words.length == 1
+      @@loot_opts.option_keys.select { |opt| opt.start_with?(str) }
+    end
+  end
+
   def cmd_loot_help
     print_line "Usage: loot [options]"
     print_line " Info: loot [-h] [addr1 addr2 ...] [-t <type1,type2>]"
     print_line "  Add: loot -f [fname] -i [info] -a [addr1 addr2 ...] -t [type]"
     print_line "  Del: loot -d [addr1 addr2 ...]"
     print_line
-    print_line "  -a,--add          Add loot to the list of addresses, instead of listing"
-    print_line "  -d,--delete       Delete *all* loot matching host and type"
-    print_line "  -f,--file         File with contents of the loot to add"
-    print_line "  -i,--info         Info of the loot to add"
-    print_line "  -t <type1,type2>  Search for a list of types"
-    print_line "  -h,--help         Show this help information"
-    print_line "  -S,--search       Search string to filter by"
+    print @@loot_opts.usage
     print_line
   end
 
+  @@loot_opts = Rex::Parser::Arguments.new(
+    [ '-a', '--add' ] => [ false, 'Add loot to the list of addresses, instead of listing.' ],
+    [ '-d', '--delete' ] => [ false, 'Delete *all* loot matching host and type.' ],
+    [ '-f', '--file' ] => [ true, 'File with contents of the loot to add.', '<filename>' ],
+    [ '-i', '--info' ] => [ true, 'Info of the loot to add.', '<info>' ],
+    [ '-t', '--type' ] => [ true, 'Search for a list of types.', '<type1,type2>' ],
+    [ '-h', '--help' ] => [ false, 'Show this help information.' ],
+    [ '-S', '--search' ] => [ true, 'Search string to filter by.', '<filter>' ],
+    [ '-u', '--update' ] => [ false, 'Update loot. Not officially supported.' ]
+  )
+
   def cmd_loot(*args)
     return unless active?
 
@@ -1222,48 +1376,49 @@ class Db
     file = nil
     name = nil
     info = nil
+    filename = nil
 
-    while (arg = args.shift)
-      case arg
-        when '-a','--add'
-          mode = :add
-        when '-d','--delete'
-          mode = :delete
-        when '-f','--file'
-          filename = args.shift
-          if(!filename)
-            print_error("Can't make loot with no filename")
-            return
-          end
-          if (!File.exist?(filename) or !File.readable?(filename))
-            print_error("Can't read file")
-            return
-          end
-        when '-i','--info'
-          info = args.shift
-          if(!info)
-            print_error("Can't make loot with no info")
-            return
-          end
-        when '-t', '--type'
-          typelist = args.shift
-          if(!typelist)
-            print_error("Invalid type list")
-            return
-          end
-          types = typelist.strip().split(",")
-        when '-S', '--search'
-          search_term = args.shift
-        when '-u', '--update' # TODO: This is currently undocumented because it's not officially supported.
-          mode = :update
-        when '-h','--help'
-          cmd_loot_help
+    @@loot_opts.parse(args) do |opt, idx, val|
+      case opt
+      when '-a', '--add'
+        mode = :add
+      when '-d', '--delete'
+        mode = :delete
+      when '-f', '--file'
+        filename = val
+        if(!filename)
+          print_error("Can't make loot with no filename")
           return
-        else
-          # Anything that wasn't an option is a host to search for
-          unless (arg_host_range(arg, host_ranges))
-            return
-          end
+        end
+        if (!File.exist?(filename) or !File.readable?(filename))
+          print_error("Can't read file")
+          return
+        end
+      when '-i', '--info'
+        info = val
+        if(!info)
+          print_error("Can't make loot with no info")
+          return
+        end
+      when '-t', '--type'
+        typelist = val
+        if(!typelist)
+          print_error("Invalid type list")
+          return
+        end
+        types = typelist.strip().split(",")
+      when '-S', '--search'
+        search_term = val
+      when '-u', '--update' # TODO: This is currently undocumented because it's not officially supported.
+        mode = :update
+      when '-h', '--help'
+        cmd_loot_help
+        return
+      else
+        # Anything that wasn't an option is a host to search for
+        unless (arg_host_range(val, host_ranges))
+          return
+        end
       end
     end
 
@@ -1704,6 +1859,7 @@ class Db
     end
   end
 
+
   def cmd_db_connect_help
     print_line("   USAGE:")
     print_line("      * Postgres Data Service:")
@@ -1744,26 +1900,26 @@ class Db
     opts = {}
     while (arg = args.shift)
       case arg
-        when '-h', '--help'
-          cmd_db_connect_help
+      when '-h', '--help'
+        cmd_db_connect_help
+        return
+      when '-y', '--yaml'
+        opts[:yaml_file] = args.shift
+      when '-c', '--cert'
+        opts[:cert] = args.shift
+      when '-t', '--token'
+        opts[:api_token] = args.shift
+      when '-l', '--list-services'
+        list_saved_data_services
+        return
+      when '-n', '--name'
+        opts[:name] = args.shift
+        if opts[:name] =~ /\/|\[|\]/
+          print_error "Provided name contains an invalid character. Aborting connection."
           return
-        when '-y', '--yaml'
-          opts[:yaml_file] = args.shift
-        when '-c', '--cert'
-          opts[:cert] = args.shift
-        when '-t', '--token'
-          opts[:api_token] = args.shift
-        when '-l', '--list-services'
-          list_saved_data_services
-          return
-        when '-n', '--name'
-          opts[:name] = args.shift
-          if opts[:name] =~ /\/|\[|\]/
-            print_error "Provided name contains an invalid character. Aborting connection."
-            return
-          end
-        when '--skip-verify'
-          opts[:skip_verify] = true
+        end
+      when '--skip-verify'
+        opts[:skip_verify] = true
       else
         found_name = ::Msf::DbConnector.data_service_search(name: arg)
         if found_name
