@@ -3,6 +3,7 @@
 require 'rex/post/meterpreter/command_mapper'
 require 'rex/post/meterpreter/packet_response_waiter'
 require 'rex/exceptions'
+require 'pathname'
 
 module Rex
 module Post
@@ -130,8 +131,7 @@ module PacketDispatcher
       tlv_enc_key = opts[:tlv_enc_key]
     end
 
-    # Uncomment this line if you want to see outbound packets in the console.
-    #STDERR.puts("SEND: #{packet.inspect}\n")
+    log_packet(packet, :send)
 
     bytes = 0
     raw   = packet.to_r(session_guid, tlv_enc_key)
@@ -178,7 +178,7 @@ module PacketDispatcher
     if timeout.nil?
       return nil
     elsif response.nil?
-      raise TimeoutError.new("Send timed out")
+      raise Rex::TimeoutError.new("Send timed out")
     elsif (response.result != 0)
       einfo = lookup_error(response.result)
       e = RequestError.new(packet.method, einfo, response.result)
@@ -579,8 +579,7 @@ module PacketDispatcher
   def dispatch_inbound_packet(packet)
     handled = false
 
-    # Uncomment this line if you want to see inbound packets in the console
-    #STDERR.puts("RECV: #{packet.inspect}\n")
+    log_packet(packet, :recv)
 
     # Update our last reply time
     self.last_checkin = ::Time.now
@@ -634,11 +633,80 @@ module PacketDispatcher
     @inbound_handlers.delete(handler)
   end
 
+  def initialize_tlv_logging(opt)
+    self.tlv_logging_error_occured = false
+    self.tlv_log_file = nil
+    self.tlv_log_file_path = nil
+    self.tlv_log_output = :none
+
+    if opt.casecmp?('console') || opt.casecmp?('true')
+      self.tlv_log_output = :console
+    elsif opt.start_with?('file:')
+      self.tlv_log_output = :file
+      self.tlv_log_file_path = opt.split('file:').last
+    end
+  end
+
 protected
 
   attr_accessor :receiver_thread # :nodoc:
   attr_accessor :dispatcher_thread # :nodoc:
   attr_accessor :waiters # :nodoc:
+
+  attr_accessor :tlv_log_output # :nodoc:
+  attr_accessor :tlv_log_file # :nodoc:
+  attr_accessor :tlv_log_file_path # :nodoc:
+  attr_accessor :tlv_logging_error_occured # :nodoc:
+
+  def shutdown_tlv_logging
+    self.tlv_log_output = :none
+    self.tlv_log_file.close unless self.tlv_log_file.nil?
+    self.tlv_log_file = nil
+    self.tlv_log_file_path = nil
+  end
+
+  def log_packet(packet, packet_type)
+    # if we previously failed to log, return
+    return if self.tlv_logging_error_occured || self.tlv_log_output == :none
+
+    if self.tlv_log_output == :console
+      log_packet_to_console(packet, packet_type)
+    elsif self.tlv_log_output == :file
+      log_packet_to_file(packet, packet_type)
+    end
+  end
+
+  def log_packet_to_console(packet, packet_type)
+    if packet_type == :send
+      print "\n%redSEND%clr: #{packet.inspect}\n"
+    elsif packet_type == :recv
+      print "\n%bluRECV%clr: #{packet.inspect}\n"
+    end
+  end
+
+  def log_packet_to_file(packet, packet_type)
+    pathname = ::Pathname.new(self.tlv_log_file_path.split('file:').last)
+
+    begin
+      if self.tlv_log_file.nil? || self.tlv_log_file.path != pathname.to_s
+        self.tlv_log_file.close unless self.tlv_log_file.nil?
+
+        self.tlv_log_file = ::File.open(pathname, 'a+')
+      end
+
+      if packet_type == :recv
+        self.tlv_log_file.puts("\nRECV: #{packet.inspect}\n")
+      elsif packet_type == :send
+        self.tlv_log_file.puts("\nSEND: #{packet.inspect}\n")
+      end
+    rescue ::StandardError => e
+      self.tlv_logging_error_occured = true
+      print_error "Failed writing to TLV Log File: #{pathname} with error: #{e.message}. Turning off logging for this session: #{self.inspect}..."
+      elog(e)
+      shutdown_tlv_logging
+      return
+    end
+  end
 end
 
 module HttpPacketDispatcher
@@ -653,6 +721,9 @@ module HttpPacketDispatcher
       'Proc'             => Proc.new { |cli, req| on_passive_request(cli, req) },
       'VirtualDirectory' => true
     )
+
+    # Add a reference count to the handler
+    self.passive_service.ref
   end
 
   def shutdown_passive_dispatcher
@@ -661,9 +732,7 @@ module HttpPacketDispatcher
       resource_uri = "/" + self.conn_id.to_s.gsub(/(^\/|\/$)/, '') + "/"
       self.passive_service.remove_resource(resource_uri) if self.passive_service
 
-      if self.passive_service.resources.empty?
-        Rex::ServiceManager.stop_service(self.passive_service)
-      end
+      self.passive_service.deref
       self.passive_service = nil
     end
     super
@@ -708,4 +777,3 @@ module HttpPacketDispatcher
 end
 
 end; end; end
-
