@@ -166,7 +166,7 @@ class Console::CommandDispatcher::Core
   end
 
   def cmd_pivot_tabs(str, words)
-    return %w[list add remove] + @@pivot_opts.fmt.keys if words.length == 1
+    return %w[list add remove] + @@pivot_opts.option_keys if words.length == 1
 
     case words[-1]
     when '-a'
@@ -180,7 +180,7 @@ class Console::CommandDispatcher::Core
     when '-t'
       return ['pipe']
     when 'add', 'remove'
-      return @@pivot_opts.fmt.keys
+      return @@pivot_opts.option_keys
     end
 
     []
@@ -350,6 +350,7 @@ class Console::CommandDispatcher::Core
   @@channel_opts = Rex::Parser::Arguments.new(
     '-c' => [ true,  'Close the given channel.' ],
     '-k' => [ true,  'Close the given channel.' ],
+    '-K' => [ false, 'Close all channels.' ],
     '-i' => [ true,  'Interact with the given channel.' ],
     '-l' => [ false, 'List active channels.' ],
     '-r' => [ true,  'Read from the given channel.' ],
@@ -392,6 +393,8 @@ class Console::CommandDispatcher::Core
       when '-w'
         mode = :write
         chan = val
+      when '-K'
+        mode = :kill_all
       end
 
       if @@channel_opts.arg_required?(opt)
@@ -427,6 +430,19 @@ class Console::CommandDispatcher::Core
       cmd_read(chan)
     when :write
       cmd_write(chan)
+    when :kill_all
+      if client.channels.empty?
+        print_line('No active channels.')
+        return
+      end
+
+      print_line('Killing all channels...')
+      client.channels.each_pair do |id, channel|
+        channel._close
+      rescue ::StandardError
+        print_error("Failed when trying to kill channel: #{id}")
+      end
+      print_line('Killed all channels.')
     else
       # No mode, no service.
       return true
@@ -436,7 +452,7 @@ class Console::CommandDispatcher::Core
   def cmd_channel_tabs(str, words)
     case words.length
     when 1
-      @@channel_opts.fmt.keys
+      @@channel_opts.option_keys
     when 2
       case words[1]
       when '-k', '-c', '-i', '-r', '-w'
@@ -558,7 +574,7 @@ class Console::CommandDispatcher::Core
 
   def cmd_irb_tabs(str, words)
     return [] if words.length > 1
-    @@irb_opts.fmt.keys
+    @@irb_opts.option_keys
   end
 
   #
@@ -645,7 +661,7 @@ class Console::CommandDispatcher::Core
 
   def cmd_set_timeouts_tabs(str, words)
     return [] if words.length > 1
-    @@set_timeouts_opts.fmt.keys
+    @@set_timeouts_opts.option_keys
   end
 
   def cmd_set_timeouts(*args)
@@ -887,7 +903,7 @@ class Console::CommandDispatcher::Core
   end
 
   def cmd_transport_tabs(str, words)
-    return %w[list change add next prev remove] + @@transport_opts.fmt.keys if words.length == 1
+    return %w[list change add next prev remove] + @@transport_opts.option_keys if words.length == 1
 
     case words[-1]
     when '-c'
@@ -899,7 +915,7 @@ class Console::CommandDispatcher::Core
     when '-t'
       return %w[reverse_tcp reverse_http reverse_https bind_tcp]
     when 'add', 'remove', 'change'
-      return @@transport_opts.fmt.keys
+      return @@transport_opts.option_keys
     end
 
     []
@@ -1192,7 +1208,7 @@ class Console::CommandDispatcher::Core
 
     begin
       server = client.sys.process.open
-    rescue TimeoutError => e
+    rescue Rex::TimeoutError, ::Timeout::Error => e
       elog('Server Timeout', error: e)
     rescue RequestError => e
       elog('Request Error', error: e)
@@ -1270,13 +1286,14 @@ class Console::CommandDispatcher::Core
     @@load_opts.parse(args) { |opt, idx, val|
       case opt
       when '-l'
-        exts = SortedSet.new
+        exts = Set.new
         if extensions.include?('stdapi') && !client.sys.config.sysinfo['BuildTuple'].blank?
           # Use API to get list of extensions from the gem
           exts.merge(MetasploitPayloads::Mettle.available_extensions(client.sys.config.sysinfo['BuildTuple']))
         else
           exts.merge(client.binary_suffix.map { |suffix| MetasploitPayloads.list_meterpreter_extensions(suffix) }.flatten)
         end
+        exts = exts.sort.uniq
         print(exts.to_a.join("\n") + "\n")
 
         return true
@@ -1320,13 +1337,14 @@ class Console::CommandDispatcher::Core
         if (client.core.use(modulenameprovided) == true)
           add_extension_client(md)
 
-          if md == 'stdapi' && !client.exploit_datastore['AutoLoadStdapi'] && client.exploit_datastore['AutoSystemInfo']
+          if md == 'stdapi' && (client.exploit_datastore && !client.exploit_datastore['AutoLoadStdapi'] && client.exploit_datastore['AutoSystemInfo'])
             client.load_session_info
           end
         end
       rescue => ex
         print_line
         log_error("Failed to load extension: #{ex.message}")
+        elog(ex)
         if ex.kind_of?(ExtensionLoadError) && ex.name
           # MetasploitPayloads and MetasploitPayloads::Mettle do things completely differently, build an array of
           # suggestion keys (binary_suffixes and Mettle build-tuples)
@@ -1374,12 +1392,13 @@ class Console::CommandDispatcher::Core
   end
 
   def cmd_load_tabs(str, words)
-    tabs = SortedSet.new
+    tabs = Set.new
     if extensions.include?('stdapi') && !client.sys.config.sysinfo['BuildTuple'].blank?
       tabs.merge(MetasploitPayloads::Mettle.available_extensions(client.sys.config.sysinfo['BuildTuple']))
     else
       tabs.merge(client.binary_suffix.map { |suffix| MetasploitPayloads.list_meterpreter_extensions(suffix) }.flatten)
     end
+    tabs = tabs.sort.uniq
     return tabs.to_a
   end
 
