@@ -18,12 +18,10 @@ module Http
 ###
 class Client
 
-  DefaultUserAgent = ClientRequest::DefaultUserAgent
-
   #
   # Creates a new client instance
   #
-  def initialize(host, port = 80, context = {}, ssl = nil, ssl_version = nil, proxies = nil, username = '', password = '')
+  def initialize(host, port = 80, context = {}, ssl = nil, ssl_version = nil, proxies = nil, username = '', password = '', comm: nil)
     self.hostname = host
     self.port     = port.to_i
     self.context  = context
@@ -32,12 +30,15 @@ class Client
     self.proxies  = proxies
     self.username = username
     self.password = password
+    self.comm = comm
 
     # Take ClientRequest's defaults, but override with our own
     self.config = Http::ClientRequest::DefaultConfig.merge({
       'read_max_data'   => (1024*1024*1),
       'vhost'           => self.hostname,
+      'ssl_server_name_indication' => self.hostname,
     })
+    self.config['agent'] ||= Rex::UserAgent.session_agent
 
     # XXX: This info should all be controlled by ClientRequest
     self.config_types = {
@@ -62,14 +63,14 @@ class Client
       'pad_get_params_count'   => 'integer',
       'pad_post_params'        => 'bool',
       'pad_post_params_count'  => 'integer',
+      'shuffle_get_params'     => 'bool',
+      'shuffle_post_params'    => 'bool',
       'uri_fake_end'           => 'bool',
       'uri_fake_params_start'  => 'bool',
       'header_folding'         => 'bool',
       'chunked_size'           => 'integer',
       'partial'                => 'bool'
     }
-
-
   end
 
   #
@@ -141,6 +142,7 @@ class Client
   # @option opts 'encode_params' [Bool]   URI encode the GET or POST variables (names and values), default: true
   # @option opts 'vars_get'      [Hash]   GET variables as a hash to be translated into a query string
   # @option opts 'vars_post'     [Hash]   POST variables as a hash to be translated into POST data
+  # @option opts 'vars_form_data'     [Hash]   POST form_data variables as a hash to be translated into multi-part POST form data
   #
   # @return [ClientRequest]
   def request_cgi(opts = {})
@@ -149,7 +151,6 @@ class Client
     opts['cgi'] = true
     opts['port'] = self.port
     opts['ssl'] = self.ssl
-    opts['ctype'] ||= 'application/x-www-form-urlencoded' if opts['method'] == 'POST'
 
     ClientRequest.new(opts)
   end
@@ -173,7 +174,8 @@ class Client
     timeout = (t.nil? or t == -1) ? 0 : t
 
     self.conn = Rex::Socket::Tcp.create(
-      'PeerHost'   => self.hostname,
+      'PeerHost'    => self.hostname,
+      'PeerHostname' => self.config['ssl_server_name_indication'] || self.config['vhost'],
       'PeerPort'   => self.port.to_i,
       'LocalHost'  => self.local_host,
       'LocalPort'  => self.local_port,
@@ -181,7 +183,8 @@ class Client
       'SSL'        => self.ssl,
       'SSLVersion' => self.ssl_version,
       'Proxies'    => self.proxies,
-      'Timeout'    => timeout
+      'Timeout'    => timeout,
+      'Comm'       => self.comm
     )
   end
 
@@ -225,14 +228,14 @@ class Client
   # @return (see #read_response)
   def _send_recv(req, t = -1, persist = false)
     @pipeline = persist
-    if req.opts['ntlm_transform_request'] and self.ntlm_client
+    if req.respond_to?(:opts) && req.opts['ntlm_transform_request'] && self.ntlm_client
       req = req.opts['ntlm_transform_request'].call(self.ntlm_client, req)
     end
 
     send_request(req, t)
-    
+
     res = read_response(t)
-    if req.opts['ntlm_transform_response'] and self.ntlm_client
+    if req.respond_to?(:opts) && req.opts['ntlm_transform_response'] && self.ntlm_client
       req.opts['ntlm_transform_response'].call(self.ntlm_client, res)
     end
     res.request = req.to_s if res
@@ -599,8 +602,8 @@ class Client
           rblob = rbody.to_s + rbufq.to_s
           tries = 0
           begin
-            # XXX: This doesn't deal with chunked encoding or "Content-type: text/html; charset=..."
-            while tries < 1000 and resp.headers["Content-Type"]== "text/html" and rblob !~ /<\/html>/i
+            # XXX: This doesn't deal with chunked encoding
+            while tries < 1000 and resp.headers["Content-Type"] and resp.headers["Content-Type"].start_with?('text/html') and rblob !~ /<\/html>/i
               buff = conn.get_once(-1, 0.05)
               break if not buff
               rblob += buff
@@ -677,6 +680,10 @@ class Client
     nil
   end
 
+  #
+  # An optional comm to use for creating the underlying socket.
+  #
+  attr_accessor :comm
   #
   # The client request configuration
   #
@@ -727,7 +734,6 @@ protected
   # The established NTLM connection info
   #
   attr_accessor :ntlm_client
-
 end
 
 end
