@@ -71,7 +71,15 @@ class MetasploitModule < Msf::Exploit::Local
                      'doubleput.c'
                    ]
         },
-        'DefaultTarget' => 1
+        'DefaultTarget' => 1,
+        'Compat' => {
+          'Meterpreter' => {
+            'Commands' => %w[
+              stdapi_fs_delete_file
+              stdapi_sys_process_execute
+            ]
+          }
+        }
       )
     )
     register_options [
